@@ -34,8 +34,8 @@
 
 #include "qmi.h"
 #include "nas.h"
+#include "util.h"
 
-#include "qmimodem.h"
 #include "src/common.h"
 
 struct netreg_data {
@@ -45,6 +45,10 @@ struct netreg_data {
 	int lac;
 	int cellid;
 	bool is_roaming;
+	uint16_t event_indication_id;
+	uint16_t signal_info_indication_id;
+	uint16_t system_info_indication_id;
+	uint16_t serving_system_indication_id;
 };
 
 enum roaming_status {
@@ -261,7 +265,7 @@ static void qmi_registration_status(struct ofono_netreg *netreg,
 
 	cbd->user = data;
 
-	if (qmi_service_send(data->nas, QMI_NAS_GET_SS_INFO, NULL,
+	if (qmi_service_send(data->nas, QMI_NAS_GET_SERVING_SYSTEM, NULL,
 					get_ss_info_cb, cbd, g_free) > 0)
 		return;
 
@@ -379,7 +383,7 @@ static void qmi_list_operators(struct ofono_netreg *netreg,
 
 	DBG("");
 
-	if (qmi_service_send(data->nas, QMI_NAS_SCAN_NETS, NULL,
+	if (qmi_service_send(data->nas, QMI_NAS_NETWORK_SCAN, NULL,
 					scan_nets_cb, cbd, g_free) > 0)
 		return;
 
@@ -423,18 +427,13 @@ static void qmi_register_auto(struct ofono_netreg *netreg,
 
 	param = qmi_param_new_uint8(QMI_NAS_PARAM_REGISTER_ACTION,
 					QMI_NAS_REGISTER_ACTION_AUTO);
-	if (!param)
-		goto error;
 
-	if (qmi_service_send(data->nas, QMI_NAS_REGISTER_NET, param,
+	if (qmi_service_send(data->nas, QMI_NAS_NETWORK_REGISTER, param,
 					register_net_cb, cbd, g_free) > 0)
 		return;
 
 	qmi_param_free(param);
-
-error:
 	CALLBACK_WITH_FAILURE(cb, cbd->data);
-
 	g_free(cbd);
 }
 
@@ -451,8 +450,6 @@ static void qmi_register_manual(struct ofono_netreg *netreg,
 
 	param = qmi_param_new_uint8(QMI_NAS_PARAM_REGISTER_ACTION,
 					QMI_NAS_REGISTER_ACTION_MANUAL);
-	if (!param)
-		goto error;
 
 	info.mcc = atoi(mcc);
 	info.mnc = atoi(mnc);
@@ -461,15 +458,12 @@ static void qmi_register_manual(struct ofono_netreg *netreg,
 	qmi_param_append(param, QMI_NAS_PARAM_REGISTER_MANUAL_INFO,
 						sizeof(info), &info);
 
-	if (qmi_service_send(data->nas, QMI_NAS_REGISTER_NET, param,
+	if (qmi_service_send(data->nas, QMI_NAS_NETWORK_REGISTER, param,
 					register_net_cb, cbd, g_free) > 0)
 		return;
 
 	qmi_param_free(param);
-
-error:
 	CALLBACK_WITH_FAILURE(cb, cbd->data);
-
 	g_free(cbd);
 }
 
@@ -527,7 +521,7 @@ static void qmi_signal_strength(struct ofono_netreg *netreg,
 
 	DBG("");
 
-	if (qmi_service_send(data->nas, QMI_NAS_GET_RSSI, NULL,
+	if (qmi_service_send(data->nas, QMI_NAS_GET_SIGNAL_STRENGTH, NULL,
 					get_rssi_cb, cbd, g_free) > 0)
 		return;
 
@@ -536,6 +530,20 @@ static void qmi_signal_strength(struct ofono_netreg *netreg,
 	g_free(cbd);
 }
 
+static void system_info_notify(struct qmi_result *result, void *user_data)
+{
+	DBG("");
+
+	qmi_result_print_tlvs(result);
+}
+
+static void signal_info_notify(struct qmi_result *result, void *user_data)
+{
+	DBG("");
+
+	qmi_result_print_tlvs(result);
+}
+
 static void event_notify(struct qmi_result *result, void *user_data)
 {
 	struct ofono_netreg *netreg = user_data;
@@ -571,20 +579,66 @@ static void event_notify(struct qmi_result *result, void *user_data)
 	}
 }
 
-static void set_event_cb(struct qmi_result *result, void *user_data)
+static void register_indications_cb(struct qmi_result *result,
+							void *user_data)
 {
 	struct ofono_netreg *netreg = user_data;
 	struct netreg_data *data = ofono_netreg_get_data(netreg);
 
-	DBG("");
+	if (qmi_result_set_error(result, NULL)) {
+		ofono_netreg_remove(netreg);
+		return;
+	}
 
 	ofono_netreg_register(netreg);
 
-	qmi_service_register(data->nas, QMI_NAS_EVENT,
+	data->event_indication_id =
+		qmi_service_register(data->nas, QMI_NAS_EVENT_REPORT,
 					event_notify, netreg, NULL);
 
-	qmi_service_register(data->nas, QMI_NAS_SS_INFO_IND,
+	data->serving_system_indication_id =
+		qmi_service_register(data->nas,
+					QMI_NAS_SERVING_SYSTEM_INDICATION,
 					ss_info_notify, netreg, NULL);
+
+	data->system_info_indication_id =
+		qmi_service_register(data->nas,
+					QMI_NAS_SYSTEM_INFO_INDICATION,
+					system_info_notify, netreg, NULL);
+
+	data->signal_info_indication_id =
+		qmi_service_register(data->nas,
+					QMI_NAS_SIGNAL_INFO_INDICATION,
+					signal_info_notify, netreg, NULL);
+}
+
+static void set_event_report_cb(struct qmi_result *result, void *user_data)
+{
+	struct ofono_netreg *netreg = user_data;
+	struct netreg_data *data = ofono_netreg_get_data(netreg);
+	static const uint8_t PARAM_SERVING_SYSTEM_EVENTS = 0x13;
+	static const uint8_t PARAM_SYSTEM_INFO = 0x18;
+	static const uint8_t PARAM_SIGNAL_INFO = 0x19;
+	struct qmi_param *param;
+
+	DBG("");
+
+	if (qmi_result_set_error(result, NULL))
+		goto error;
+
+	param = qmi_param_new();
+
+	qmi_param_append_uint8(param, PARAM_SERVING_SYSTEM_EVENTS, 0x01);
+	qmi_param_append_uint8(param, PARAM_SYSTEM_INFO, 0x01);
+	qmi_param_append_uint8(param, PARAM_SIGNAL_INFO, 0x01);
+
+	if (qmi_service_send(data->nas, QMI_NAS_REGISTER_INDICATIONS, param,
+				register_indications_cb, netreg, NULL) > 0)
+		return;
+
+	qmi_param_free(param);
+error:
+	ofono_netreg_remove(netreg);
 }
 
 static void create_nas_cb(struct qmi_service *service, void *user_data)
@@ -592,36 +646,37 @@ static void create_nas_cb(struct qmi_service *service, void *user_data)
 	struct ofono_netreg *netreg = user_data;
 	struct netreg_data *data = ofono_netreg_get_data(netreg);
 	struct qmi_param *param;
-	struct qmi_nas_param_event_signal_strength ss = { .report = 0x01,
-				.count = 5, .dbm[0] = -55, .dbm[1] = -65,
-				.dbm[2] = -75, .dbm[3] = -85, .dbm[4] = -95 };
+	static const uint8_t PARAM_REPORT_SIGNAL_STRENGTH = 0x10;
+	static const uint8_t PARAM_REPORT_RF_INFO = 0x11;
+	struct {
+		uint8_t report;					/* bool */
+		uint8_t count;
+		int8_t dbm[5];
+	} __attribute__((__packed__)) ss = { .report = 0x01,
+			.count = 5, .dbm[0] = -55, .dbm[1] = -65,
+			.dbm[2] = -75, .dbm[3] = -85, .dbm[4] = -95 };
 
 	DBG("");
 
 	if (!service) {
 		ofono_error("Failed to request NAS service");
-		ofono_netreg_remove(netreg);
-		return;
+		goto error;
 	}
 
 	data->nas = qmi_service_ref(service);
 
 	param = qmi_param_new();
-	if (!param)
-		goto done;
 
-	qmi_param_append(param, QMI_NAS_PARAM_REPORT_SIGNAL_STRENGTH,
-							sizeof(ss), &ss);
-	qmi_param_append_uint8(param, QMI_NAS_PARAM_REPORT_RF_INFO, 0x01);
+	qmi_param_append(param, PARAM_REPORT_SIGNAL_STRENGTH, sizeof(ss), &ss);
+	qmi_param_append_uint8(param, PARAM_REPORT_RF_INFO, 0x01);
 
-	if (qmi_service_send(data->nas, QMI_NAS_SET_EVENT, param,
-					set_event_cb, netreg, NULL) > 0)
+	if (qmi_service_send(data->nas, QMI_NAS_SET_EVENT_REPORT, param,
+					set_event_report_cb, netreg, NULL) > 0)
 		return;
 
 	qmi_param_free(param);
-
-done:
-	ofono_netreg_register(netreg);
+error:
+	ofono_netreg_remove(netreg);
 }
 
 static int qmi_netreg_probe(struct ofono_netreg *netreg,
@@ -661,7 +716,28 @@ static void qmi_netreg_remove(struct ofono_netreg *netreg)
 
 	ofono_netreg_set_data(netreg, NULL);
 
-	qmi_service_unregister_all(data->nas);
+	if (data->event_indication_id) {
+		qmi_service_unregister(data->nas, data->event_indication_id);
+		data->event_indication_id = 0;
+	}
+
+	if (data->serving_system_indication_id) {
+		qmi_service_unregister(data->nas,
+					data->serving_system_indication_id);
+		data->serving_system_indication_id = 0;
+	}
+
+	if (data->system_info_indication_id) {
+		qmi_service_unregister(data->nas,
+					data->system_info_indication_id);
+		data->system_info_indication_id = 0;
+	}
+
+	if (data->signal_info_indication_id) {
+		qmi_service_unregister(data->nas,
+					data->signal_info_indication_id);
+		data->signal_info_indication_id = 0;
+	}
 
 	qmi_service_unref(data->nas);
 
@@ -669,7 +745,6 @@ static void qmi_netreg_remove(struct ofono_netreg *netreg)
 }
 
 static const struct ofono_netreg_driver driver = {
-	.name			= "qmimodem",
 	.probe			= qmi_netreg_probe,
 	.remove			= qmi_netreg_remove,
 	.registration_status	= qmi_registration_status,
@@ -680,12 +755,4 @@ static const struct ofono_netreg_driver driver = {
 	.strength		= qmi_signal_strength,
 };
 
-void qmi_netreg_init(void)
-{
-	ofono_netreg_driver_register(&driver);
-}
-
-void qmi_netreg_exit(void)
-{
-	ofono_netreg_driver_unregister(&driver);
-}
+OFONO_ATOM_DRIVER_BUILTIN(netreg, qmimodem, &driver)
