@@ -52,7 +52,6 @@ typedef void (*ofono_gprs_context_cb_t)(const struct ofono_error *error,
 					void *data);
 
 struct ofono_gprs_context_driver {
-	const char *name;
 	int (*probe)(struct ofono_gprs_context *gc, unsigned int vendor,
 			void *data);
 	void (*remove)(struct ofono_gprs_context *gc);
@@ -72,11 +71,6 @@ struct ofono_gprs_context_driver {
 void ofono_gprs_context_deactivated(struct ofono_gprs_context *gc,
 					unsigned int id);
 
-int ofono_gprs_context_driver_register(
-				const struct ofono_gprs_context_driver *d);
-void ofono_gprs_context_driver_unregister(
-				const struct ofono_gprs_context_driver *d);
-
 struct ofono_gprs_context *ofono_gprs_context_create(struct ofono_modem *modem,
 						unsigned int vendor,
 						const char *driver, void *data);
